class BaseBackend(object):
    bucket_prefix = 'ONTHEFLY'

    def __init__(self, options, original=None):
        self.options = options
        self.original = original
        self._all_fields = None

    @property
    def all_fields(self):
        if not self._all_fields:
            self._all_fields = self.get_all_fields()
        return self._all_fields

    def add_field(self, name):
        if name not in self.all_fields:
            self._all_fields.append(name)
            self.set_fields()
            original_value = self.get_value_from_original_settings(name)
            self.set_value(name, original_value)

    def delete_field(self, name):
        if name in self._all_fields:
            self._all_fields.remove(name)
            self.set_fields()
            self.delete_value(name)

    def get_value_from_original_settings(self, name):
        return getattr(self.original, name)

    def set_fields(self):
        raise NotImplementedError()

    def set_value(self, name, value):
        raise NotImplementedError()

    def get_value(self, name):
        raise NotImplementedError()

    def get_all_values(self):
        raise NotImplementedError()

    def delete_value(self, name):
        raise NotImplementedError()



from django.apps import AppConfig

from onthefly.monkey_patch import patch


class OntheflyConfig(AppConfig):
    name = 'onthefly'
    patched = False

    def ready(self):
        if not self.patched:
            patch()
            self.patched = True



from django import conf
from django.views.debug import get_safe_settings
from onthefly.utils import load_class


class OnTheFlySettings(object):
    def __init__(self, decoratee):
        self._decoratee = decoratee
        configuration = getattr(self._decoratee, 'ONTHEFLY', {
            'BACKEND': 'onthefly.backend.redis_backend.RedisBackend',
            'OPTIONS': {'URL': 'redis://localhost:6379/15'}})
        backend = configuration['BACKEND']
        options = configuration['OPTIONS']
        backend_class = load_class(backend)
        self.backend = backend_class(options, original=decoratee)

    def __getattr__(self, name):
        if name in self.backend.all_fields:
            return self.backend.get_value(name)
        else:
            return getattr(self._decoratee, name)

    @property
    def __module__(self):
        return self._decoratee.__module__

    @property
    def get_original_settings_without_onthefly(self):
        all_settings = get_safe_settings().keys()
        original_settings_without_onthefly = set(all_settings).difference(
            self.backend.all_fields)
        return list(original_settings_without_onthefly)

    @property
    def get_onthefly_settings(self):
        return self.backend.get_all_values()


def patch():
    conf.settings = OnTheFlySettings(conf.settings)



import os
import imp


def load_class(path):
    """
    Loads class from path.
    """

    mod_name, klass_name = path.rsplit('.', 1)
    mod_args = mod_name.split('.')
    mod_args.pop(0)

    try:
        import onthefly as djonthefly
        mod_path = '%s/%s/%s.py' % (
            os.path.dirname(djonthefly.__file__), mod_args[0], mod_args[1])
        mod = imp.load_source(mod_name, mod_path)
    except AttributeError as e:
        raise ImproperlyConfigured(
            'Error importing {0}: "{1}"'.format(mod_name, e))

    try:
        klass = getattr(mod, klass_name)
    except AttributeError:
        raise ImproperlyConfigured(
            'Module "{0}" does not define a "{1}" class'.format(mod_name,
                                                                klass_name))

    return klass



from django.contrib import admin
from django.views.generic import TemplateView
from django.conf import settings


class AppSettingsView(TemplateView):
    """
    Show current settings
    """
    template_name = 'admin/onthefly_settings.html'

    def get_context_data(self, *args, **kwargs):
        context = super(AppSettingsView, self).get_context_data(
            *args, **kwargs)
        context.update({
            'original_settings': (
                settings.get_original_settings_without_onthefly),
            'onthefly_settings': settings.get_onthefly_settings
        })
        return context

    def post(self, request, *args, **kwargs):
        action_type = request.POST.get('actiontype')
        name = request.POST.get('name')
        if action_type == 'add_field':
            settings.backend.add_field(name)
        elif action_type == 'delete_field':
            settings.backend.delete_field(name)
        elif action_type == 'set_value':
            settings.backend.set_value(name, request.POST.get('value'))
        request.method = 'GET'
        return self.get(request, *args, **kwargs)


admin.site.register_view('onthefly-settings/', 'Onthefly Settings',
                         view=AppSettingsView.as_view())
